##
## R code for plotting feature weights and prediction profiles for
## epitope-to-MHC binding data set as presented in the manuscript
##
##    J. Palme, S. Hochreiter, and U. Bodenhofer.
##    KeBABS: an R package for kernel-based analysis of biological sequences
##    Bioinformatics 31:2574-2576, 2015. DOI: 10.1093/bioinformatics/btv176.
##
##  NOTE: The code in this source file requires at least version 1.1.8 of
##        the 'kebabs' package.
##

## load packages
library(lattice)
library(kebabs)
packageDescription("kebabs")$Version ## NOTE: at least version 1.1.8 required!

## load R workspace file with data set
load("MHC_I_A0201_Strong.RData")

ls()
a0201strongseqs
table(a0201strongy)

## train model on full data including computation of feature weights
model <- kbsvm(a0201strongseqs, a0201strongy, spectrumKernel(k=1, distWeight=1),
               pkg="kernlab", svm="C-svc", cost=1.1, explicit="no")

## predict all sequences to get decision value and to check training performance
predall <- predict(model, a0201strongseqs)
preddecall <- predict(model, a0201strongseqs, predictionType="decision")
evaluatePrediction(predall, a0201strongy, allLabels=unique(a0201strongy),
                   decValues=preddecall)

## order by decision value
ind <- order(preddecall)

## compute prediction profiles for all samples
predprof <- getPredictionProfile(a0201strongseqs,
                                 spectrumKernel(k=1, distWeight=1),
                                 featureWeights(model), modelOffset(model))

## plot a pair of prediction profiles for one binder B241 and
##    one non-binder (NB163)
png("PredictionProfiles_B241_NB163.png", height=1000, width=950)
plot(predprof, sel=c(ind[1023], ind[27]),
     col=c(rgb(0, 0.7, 0.7), rgb(0.9, 0.35, 0.35)),
     shades=c(rgb(0.99, 0.85, 0.85), rgb(0.83, 0.92, 0.93)),
     cex=2.5, cex.lab=1.5, cex.axis=1.5,
     ylab="", lwd.profile=5, lwd.axis=3, standardize=FALSE)
dev.off()

## convert feature weights to dense matrix for levelplot
fw_matrix <- as(featureWeights(model), "matrix")
## set up color palette
colh <- colorRampPalette(c(rgb(1, 0, 0), rgb(1, 1, 1), rgb(0.47, 0.75, 0.75)))
colh1 <- colh(20)

## plot feature weights
png("FeatureWeights.png", height=1170, width=750, res=120)
levelplot(t(fw_matrix[20:1, ]), col.regions=colh1, aspect=2.3,
          xlab="Position", ylab="Feature", main="Feature Weights",
          scales=list(y=list(cex=1), x=list(cex=1),tck=c(0, 0)))
dev.off()

## get profile matrix, delete row names and adapt column names for plot
currProf <- profiles(predprof)
colnames(currProf) <- 1:9
rownames(currProf) <- NULL

## plot prediction profile of all samples sorted by decision value
png("PredictionProfilesSortedByDecVal.png", height=1400, width=1000, res=100)
levelplot(t(currProf[ind, ]), aspect=1.7,
          at=unique(c(seq(-1.22, 0, length=10), seq(0, 1.1, length=10))),
          xlab="Positions", ylab="Samples",
          main="Prediction Profiles (sorted by decision value)",
          col.regions=colorRampPalette(c("red", "white", "cadetblue3"))(20),
          scales=list(x=list(cex=1.5), y=list(cex=1.5)),
          colorkey=list(tck=2, labels=list(cex=1.5)))
dev.off()
